;==========================================================================
;       Anwendung:	    PS2-Adapter fr JU+TE mit ES2.3
;       Autor:		    E.Mueller
;       Prozessor:	    PIC16F628A
;       Takt:		    intern, 4Mhz
;       erstellt:           09.12.2015
;       letzte nderung:    07.04.2018
;==========================================================================

W                            EQU     H'0000'
F                            EQU     H'0001'
;--- Register Zellen
INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'

INTCON                       EQU     H'000B'
OPTION_R	             EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
CMCON                        EQU     H'001F'
;--- STATUS Bits
IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'
;--- RAM Definition
	__MAXRAM H'01FF'
	__BADRAM H'07'-H'09', H'0D', H'13'-H'14', H'1B'-H'1E'
	__BADRAM H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
	__BADRAM H'105', H'107'-H'109', H'10C'-H'11F', H'150'-H'16F'
	__BADRAM H'185', H'187'-H'189', H'18C'-H'1EF'

	
	
	;--- Konfigurations Bits
_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'

_CPD_ON              EQU  H'3EFF'; Data memory code-protected
DATA_CP_ON           EQU  H'3EFF'; Data memory code-protected
_CPD_OFF             EQU  H'3FFF'; Data memory code protection off
DATA_CP_OFF          EQU  H'3FFF'; Data memory code protection off

_CP_ON               EQU  H'1FFF'; 0000h to 07FFh code-protected
_CP_OFF              EQU  H'3FFF'; Code protection off

;_CP_ALL                      EQU     H'03FF'
;_CP_75                       EQU     H'17FF'
;_CP_50                       EQU     H'2BFF'
;_CP_OFF                      EQU     H'3FFF'
;_DATA_CP_ON                  EQU     H'3EFF'
;_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'
		    
		    

	__CONFIG        _BODEN_ON & _CP_ON & _CPD_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _INTRC_OSC_NOCLKOUT  


;==========================================================================
;       Makro Definition
;==========================================================================


IstNichtNull	MACRO Register, Ziel
		MOVF	Register,W
		BTFSS	STATUS,Z
		GOTO	Ziel
		ENDM

IstWertRufe	MACRO Register, Wert, Rufe
		MOVF	Register,W
		XORLW	Wert
		BTFSC	STATUS,Z
		CALL	Rufe
		ENDM

IstWert		MACRO Register, Wert, Ziel
		MOVF	Register,W
		XORLW	Wert
		BTFSC	STATUS,Z
		GOTO	Ziel
		ENDM

IstReg		MACRO RegisterA, RegisterB, Ziel
		MOVF	RegisterA,W
		XORWF	RegisterB,W
		BTFSC	STATUS,Z
		GOTO	Ziel
		ENDM


Bewerten	MACRO Register, Wert, Spalte_Zeile, Ziel
		MOVF	Register,W
		XORLW	Wert
		BTFSs	STATUS,Z
		GOTO	$+5
		movlw	Spalte_Zeile
		movwf	h'20'
		goto	Ziel
		nop
		nop
		nop
		ENDM


;==========================================================================
;       Variablen Definition
;==========================================================================
TEMP0		EQU	H'22'		;Zelle H'20' als Hilfsvariable
TEMP1		EQU	H'23'		;Zelle H'21' als Hilfsvariable
AKTUELL		equ	h'2a'
MERKER		equ	h'2b'
SCHALTER	equ	h'24'
ZAEHLER		equ	h'25'
TASTE_SHIFT	equ	h'26'
TASTE_ALTGR	equ	h'27'

PS2TAKT		EQU     H'00'		;ausgelst durch RB0 (Bit 0)
PS2DATEN	EQU     H'01'		;ausgelst durch RB1 (Bit 1)


		ORG	0		;Startadresse fr Rcksetzen


;==========================================================================
;       Hauptprogramm
;==========================================================================
RUECKSETZEN	MOVLW	B'00000111'	;Komparator(en) abschalten
		MOVWF	CMCON
		
		BSF	STATUS,RP0	;Schalte nach Speicherbank 1

                ;PIC Optionsregister setzen (Details siehe Datenblatt)
		;Bit 0,1,2 "111" Vorteiler Verhltnis 1:256
		;Bit 3       "0" Vorteiler Zeitgeber0 (TMR0) zuordnen
		;Bit 4       "1" Signalflanke am Zeitgebereingang auf fallend stellen (extern)
		;Bit 5       "0" Signalquelle fr Zeitgeber0 auf intern stellen		
		;Bit 6       "1" Signalflanke fr Unterbrechung auf steigend stellen (RB0)
		;Bit 7       "1" interne "Pull-Up" Widerstnde an Port B abschalten		
		MOVLW	B'11010111'	
		MOVWF	OPTION_R	
		
		CLRF	INTCON		;Unterbrechungsregister lschen
		
		MOVLW	B'00000000'	;Alle RB als Ausgnge definieren
		MOVWF	TRISB		
		MOVLW	B'11111011'	;Alle RA, auer RA2 als Eingnge definieren
		MOVWF	TRISA
		BCF	STATUS,RP0	;Schalte nach Speicherbank 0

		clrf	PORTB               ;Initialize PORTB
		movlw	b'00001011'
		movwf	PORTA               ;Initialize PORTA

        	clrf	MERKER
		clrf	AKTUELL
		clrf	TASTE_SHIFT
		clrf	TASTE_ALTGR
		call	NULLEWAJA

Haupt		;clrf	AKTUELL
	      	call    PS2_Get             ;Lese ein Byte vom der Tastatur
              	call    Auswerten             ;Anzeigen
		IstNichtNull	AKTUELL,Haupt_w1
		call	NULLEWAJA
		goto	Haupt
Haupt_w1	call	ES23
              	goto    Haupt                ;von vorne
        

NULLEWAJA	movlw	h'ff'
		movwf	TEMP0
		Bewerten	TEMP0,h'ff',b'00001111',JUTE_W1	;NIX!
		;Dieser Fall tritt niemals ein !!!
		return


Auswerten	movwf	AKTUELL		;aktueller Tastenkode befindet sich in "W"
		IstWert	AKTUELL,h'12',L_Shift		
		IstWert	AKTUELL,h'f0',Losgelassen
		IstWert	AKTUELL,h'e0',Sonderzeichen
		IstWert	AKTUELL,h'aa',Auf_Null		;Selbsttest i.O.
		return
Auf_Null	clrf	AKTUELL
		bcf	PORTA,2	    ;LED aus
		call	NULLEWAJA
		return

L_Shift		movlw	h'01'
		movwf	TASTE_SHIFT
		return

Alt_Gr		movlw	h'01'
		movwf	TASTE_ALTGR
		return

Sonderzeichen	call	PS2_Get
		movwf	AKTUELL		;aktueller Tastenkode befindet sich in "W"
		IstWert	AKTUELL,h'f0',Losgelassen
		IstWert	AKTUELL,h'11',Alt_Gr		;Alt Gr

		Bewerten	AKTUELL,h'70',b'11101110',JUTE_W1		;Einfg
		Bewerten	AKTUELL,h'6C',b'11101011',JUTE_W1		;POS1
		Bewerten	AKTUELL,h'71',b'11011101',JUTE_W1		;Lschen
		Bewerten	AKTUELL,h'75',b'11101101',JUTE_W1		;Pfeil hoch
		Bewerten	AKTUELL,h'72',b'11100111',JUTE_W1		;Pfeil runter
		Bewerten	AKTUELL,h'6b',b'11011011',JUTE_W1		;Pfeil links
		Bewerten	AKTUELL,h'74',b'11111011',JUTE_W1		;Pfeil rechts
		;call	PS2_Get
		return

Losgelassen	call	PS2_Get
		movwf	AKTUELL		;aktueller Tastenkode befindet sich in "W"
		IstWert	AKTUELL,h'12',L_Shift_Aus		
		IstWert	AKTUELL,h'11',Alt_Gr_Aus		
		goto	Auf_Null
L_Shift_Aus	clrf	TASTE_SHIFT
		goto	Auf_Null
Alt_Gr_Aus	clrf	TASTE_ALTGR
		goto	Auf_Null



ES23	;Port B.0=S3, Port B.1=S2, Port B.2=S1, Port B.3=S0
		;Port B.4=A3, Port B.5=A2, Port B.6=A1, Port B.7=A0

		bsf	PORTA,2		;LED an

		BTFSC	TASTE_ALTGR,0
		goto	ES23_W0


		;SHIFT-Ebene
		BTFSS	TASTE_SHIFT,0
		goto	ES23_w1

		;normale Tastaturebene im JUTE
		Bewerten	AKTUELL,h'3d',b'10100111',JUTE_W1	;/ (7)
		Bewerten	AKTUELL,h'45',b'11001101',JUTE_W1	;= (0)
		Bewerten	AKTUELL,h'41',b'10101011',JUTE_W1	;; (,)
		Bewerten	AKTUELL,h'61',b'11001110',JUTE_W1	;> (<)
		Bewerten	AKTUELL,h'5b',b'11001011',JUTE_W1	;* (+)
		;SHIFT Tastaturebene im JUTE
		Bewerten	AKTUELL,h'16',b'00011110',JUTE_W2	;! (1)
		Bewerten	AKTUELL,h'1E',b'00101110',JUTE_W2	;" (2)
		Bewerten	AKTUELL,h'25',b'01001110',JUTE_W2	;$ (4)
		Bewerten	AKTUELL,h'2e',b'01011110',JUTE_W2	;% (5)
		Bewerten	AKTUELL,h'36',b'01101110',JUTE_W2	;& (6)
		Bewerten	AKTUELL,h'3e',b'10011110',JUTE_W2	;( (8)
		Bewerten	AKTUELL,h'46',b'10101110',JUTE_W2	;) (9)
		Bewerten	AKTUELL,h'4e',b'10100111',JUTE_W2	;? ()
		Bewerten	AKTUELL,h'4a',b'10111101',JUTE_W2	;_ (-)
		Bewerten	AKTUELL,h'49',b'10101011',JUTE_W2	;: (.)



		return

ES23_W0		;ALT-GR Tastaturebene im JUTE
		Bewerten	AKTUELL,h'3e',b'10000111',JUTE_W2	;[ (8)
		Bewerten	AKTUELL,h'46',b'10010111',JUTE_W2	;] (9)
		Bewerten	AKTUELL,h'4e',b'10111011',JUTE_W2	;\ ()
		Bewerten	AKTUELL,h'15',b'10001110',JUTE_W2	;@ (Q)




		return


		;0001		1	Q	A	Y
ES23_w1		Bewerten	AKTUELL,h'16',b'00011110',JUTE_W1	;1
		Bewerten	AKTUELL,h'15',b'00011101',JUTE_W1	;Q
		Bewerten	AKTUELL,h'1C',b'00011011',JUTE_W1	;A
		Bewerten	AKTUELL,h'1a',b'00010111',JUTE_W1	;Y

		;0010		2	W	S	X
		Bewerten	AKTUELL,h'1E',b'00101110',JUTE_W1	;2
		Bewerten	AKTUELL,h'1D',b'00101101',JUTE_W1	;W
		Bewerten	AKTUELL,h'1B',b'00101011',JUTE_W1	;S
		Bewerten	AKTUELL,h'22',b'00100111',JUTE_W1	;X

		;0011		3	E	D	C
		Bewerten	AKTUELL,h'26',b'00111110',JUTE_W1	;3
		Bewerten	AKTUELL,h'24',b'00111101',JUTE_W1	;E
		Bewerten	AKTUELL,h'23',b'00111011',JUTE_W1	;D
		Bewerten	AKTUELL,h'21',b'00110111',JUTE_W1	;C

		;0100		4	R	F	V
		Bewerten	AKTUELL,h'25',b'01001110',JUTE_W1	;4
		Bewerten	AKTUELL,h'2d',b'01001101',JUTE_W1	;R
		Bewerten	AKTUELL,h'2b',b'01001011',JUTE_W1	;F
		Bewerten	AKTUELL,h'2a',b'01000111',JUTE_W1	;V

		;0101		5	T	G	B
		Bewerten	AKTUELL,h'2e',b'01011110',JUTE_W1	;5
		Bewerten	AKTUELL,h'2c',b'01011101',JUTE_W1	;T
		Bewerten	AKTUELL,h'34',b'01011011',JUTE_W1	;G
		Bewerten	AKTUELL,h'32',b'01010111',JUTE_W1	;B

		;0110		6	Z	H	N
		Bewerten	AKTUELL,h'36',b'01101110',JUTE_W1	;6
		Bewerten	AKTUELL,h'35',b'01101101',JUTE_W1	;Z
		Bewerten	AKTUELL,h'33',b'01101011',JUTE_W1	;H
		Bewerten	AKTUELL,h'31',b'01100111',JUTE_W1	;N

		;0111		7	U	J	M
		Bewerten	AKTUELL,h'3d',b'01111110',JUTE_W1	;7
		Bewerten	AKTUELL,h'3c',b'01111101',JUTE_W1	;U
		Bewerten	AKTUELL,h'3b',b'01111011',JUTE_W1	;J
		Bewerten	AKTUELL,h'3a',b'01110111',JUTE_W1	;M

		;1000		8	I	K	,
		Bewerten	AKTUELL,h'3e',b'10001110',JUTE_W1	;8
		Bewerten	AKTUELL,h'43',b'10001101',JUTE_W1	;I
		Bewerten	AKTUELL,h'42',b'10001011',JUTE_W1	;K
		Bewerten	AKTUELL,h'41',b'10000111',JUTE_W1	;,

		;1001		9	O	L	.
		Bewerten	AKTUELL,h'46',b'10011110',JUTE_W1	;9
		Bewerten	AKTUELL,h'44',b'10011101',JUTE_W1	;O
		Bewerten	AKTUELL,h'4b',b'10011011',JUTE_W1	;L
		Bewerten	AKTUELL,h'49',b'10010111',JUTE_W1	;.

		;1010		0	P	*	/
		Bewerten	AKTUELL,h'45',b'10101110',JUTE_W1	;0
		Bewerten	AKTUELL,h'4d',b'10101101',JUTE_W1	;P
		;Bewerten	AKTUELL,h'4c',b'10101011',JUTE_W1	;;	
		;Bewerten	AKTUELL,h'4a',b'10100111',JUTE_W1	;/	

		;1011
		Bewerten	AKTUELL,h'61',b'10111110',JUTE_W1	;<
		Bewerten	AKTUELL,h'4a',b'10111101',JUTE_W1	;-	
		Bewerten	AKTUELL,h'5b',b'10111011',JUTE_W1	;+	
		Bewerten	AKTUELL,h'5a',b'10110111',JUTE_W1	;ENTER	

		;1100
		;Bewerten	AKTUELL,h'  ',b'11001110',JUTE_W1	;>
		;Bewerten	AKTUELL,h'  ',b'11001101',JUTE_W1	;=	
		;Bewerten	AKTUELL,h'  ',b'11001011',JUTE_W1	;*	
		Bewerten	AKTUELL,h'29',b'11000111',JUTE_W1	;LEER

		;1101
		Bewerten	AKTUELL,h'66',b'11011110',JUTE_W1	;BS
		;Bewerten	AKTUELL,h'  ',b'11011101',JUTE_W1	;ENTF	
		;Bewerten	AKTUELL,h'  ',b'11011011',JUTE_W1	;LINKS <-	
		;Bewerten	AKTUELL,h'  ',b'11010111',JUTE_W1	;LEER  **********

		;1110
		;Bewerten	AKTUELL,h'  ',b'11101110',JUTE_W1	;EINFG
		;Bewerten	AKTUELL,h'  ',b'11101101',JUTE_W1	;OBEN	
		;Bewerten	AKTUELL,h'  ',b'11101011',JUTE_W1	;POS1	
		;Bewerten	AKTUELL,h'  ',b'11100111',JUTE_W1	;UNTEN

		;1111
		Bewerten	AKTUELL,h'76',b'11111110',JUTE_W1	;ESC
		Bewerten	AKTUELL,h'05',b'11111101',JUTE_W1	;F1	
		;Bewerten	AKTUELL,h'  ',b'11111011',JUTE_W1	;RECHTS ->
		;Bewerten	AKTUELL,h'  ',b'11110111',JUTE_W1	;ENTER  **********



		Bewerten	AKTUELL,h'0e',b'11001011',JUTE_W2	;   ^
		Bewerten	AKTUELL,h'5d',b'00111110',JUTE_W2	;   #


		return

		
JUTE_W2		call	BitDreher
		movf	h'20',W
		;Spalten
		andlw	h'0f'
		movwf	PORTB
		bsf	PORTB,6
		bsf	PORTB,7
		call TueNichts
		bcf	PORTB,6
		bcf	PORTB,7
		;Adressen
		swapf	h'20',W
		andlw	h'0f'
		movwf	PORTB
		bsf	PORTB,4
		bsf	PORTB,5
		call TueNichts
		bcf	PORTB,4
		bcf	PORTB,5
		;Shift Adresse 0, Spalte 0
		;Spalten
		movlw	h'07'
		movwf	PORTB
		bsf	PORTB,6
		call TueNichts
		bcf	PORTB,6
		;Adressen
		movlw	h'00'
		movwf	PORTB
		bsf	PORTB,4
		call TueNichts
		bcf	PORTB,4
		;Rest
		return



JUTE_W1		call	BitDreher
		movf	h'20',W
		;Spalten
		andlw	h'0f'
		movwf	PORTB
		bsf	PORTB,6
		bsf	PORTB,7
		call TueNichts
		bcf	PORTB,6
		bcf	PORTB,7
		;Adressen
		swapf	h'20',W
		andlw	h'0f'
		movwf	PORTB
		bsf	PORTB,4
		bsf	PORTB,5
		call TueNichts
		bcf	PORTB,4
		bcf	PORTB,5
		;Rest
		return


TueNichts	nop
		nop
		nop
		nop
		nop
		return

BitDreher	rlf	h'20',F
		rrf	h'2f',F
		rlf	h'20',F
		rrf	h'2f',F
		rlf	h'20',F
		rrf	h'2f',F
		rlf	h'20',F
		rrf	h'2f',F
		swapf	h'20',W
		andlw	h'0f'
		movwf	h'20'
		movf	h'2f',W
		andlw	h'f0'
		addwf	h'20',F
		return



;		** Damit's Jeder weiss !
		de	"Einsatz ausschliesslich auf eigene Gefahr !"
		de	"Keine Haftung, egal aus welchem Grund !"
		de	"E.Mueller"




;---------------------------------------------------------------------------
;  PS2_Get -By Adam C  ROM: 21             RAM: TEMP0, TEMP1
;  TIME: ---           W': -INPUT- 
;  REQ: -NONE-
;---------------------------------------------------------------------------
PS2_Get	      call        PS2_Get1            ;Get/ignore the start bit
              movlw       d'8'                ;Load Counter
              movwf       TEMP0

PS2_Get0      rrf         TEMP1, f
              call        PS2_Get1            ;Read a bit from the keyboard/mouse
              iorwf       TEMP1, f            ;TEMP1.7 = bit read from device
              iorlw       b'01111111'
              andwf       TEMP1, f
              decfsz      TEMP0, f            ;Read 8 data bits yet?
              goto        PS2_Get0            ;  No.  Get another bit.
              call        PS2_Get1            ;  Yes. Get/ignore parity bit.
              call        PS2_Get1            ;Get/ignore stop bit
              movf        TEMP1, w            ;Put result in w.
              return                          ;Done--return.

PS2_Get1      btfss       PORTA, PS2TAKT        ;Make sure clock is high.
              goto        $ - 1
              btfsc       PORTA, PS2TAKT        ;Read data on falling edge
              goto        $ - 1
              btfss       PORTA, PS2DATEN       ;Read data.
              retlw       b'00000000'         ;  w.7 = bit read.
              retlw       b'10000000'







	END
